##############################################################################
#
# Copyright (c) 2002 Ingeniweb SARL
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.0 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE
#
##############################################################################

"""
Plone Article with images and attachments and layout models
"""

from Globals import InitializeClass
from AccessControl import ClassSecurityInfo
import string
import random
import os

from Products.CMFCore.CMFCorePermissions import View, ManageProperties, ListFolderContents
from Products.CMFDefault.DublinCore import DefaultDublinCoreImpl
from Products.CMFCore.PortalContent import PortalContent
from Products.CMFCore import CMFCorePermissions
from AccessControl import Permissions, getSecurityManager, ClassSecurityInfo, Unauthorized
from OFS import Folder, Image, ObjectManager
from Products.ZAttachmentAttribute import ZAttachmentAttribute
from global_symbols import *
from ArticleCore import ArticleCore
from Products.CMFCore.utils import getToolByName
from Products.CMFCore.utils import format_stx
from Products.CMFDefault.utils import html_headcheck
from Products.CMFDefault.utils import html_headcheck
from DocumentTemplate.DT_Util import html_quote

# PloneNewsItem definition
factory_type_information = {
  'id'             : 'News Item',
  'portal_type'    : 'News Item',
  'meta_type'      : 'News Item',
  'description'    : 'News Item with images/attachments',
  'content_icon'   : 'newsitem_icon.gif',                       # included with Plone
  'product'        : 'PloneNewsItem',
  'factory'        : 'addPloneNewsItem',
  'immediate_view' : 'PloneNewsItem_edit',
  'filter_content_types' : 0,                                   # This is for folderish types
  'allowed_content_types': (),                                  # If filter_content_types is true, this field can hold allowed meta_types
  'actions': (
    {
    'id'           : 'view',
    'name'         : 'View',
    'action'       : 'PloneNewsItem_view',
    'permissions'  : (CMFCorePermissions.View, ),
    'category'      : 'object',
    },
    {
    'id'           : 'edit',
    'name'         : 'Edit',
    'action'       : 'portal_form/PloneNewsItem_editForm',
    'permissions'   : (CMFCorePermissions.ModifyPortalContent, ),
    'category'      : 'object',
    },
    {
    'id'           : 'image',
    'name'         : 'Images',
    'action'       : 'portal_form/PloneNewsItem_imageForm',
    'permissions'   : (CMFCorePermissions.ModifyPortalContent, ),
    'category'      : 'object',
    },
    {
    'id'           : 'attach',
    'name'         : 'Attachments',
    'action'       : 'portal_form/PloneNewsItem_attachmentForm',
    'permissions'   : (CMFCorePermissions.ModifyPortalContent, ),
    'category'      : 'object',
    },
    {
    'id'           : 'model',
    'name'         : 'Model',
    'action'       : 'portal_form/PloneNewsItem_modelForm',
    'permissions'   : (CMFCorePermissions.ModifyPortalContent, ),
    'category'      : 'object',
    },
    {
    'id'            : 'metadata',
    'name'          : 'Properties',
    'action'        : 'metadata_edit_form',
    'permissions'   : (CMFCorePermissions.ModifyPortalContent, ),
    'category'      : 'object',
    },
    ),
  }

factory_type_information2 = {
  'id'             : 'News Item',
  'portal_type'    : 'News Item',
  'meta_type'      : 'News Item',
  'description'    : 'News Item with images/attachments',
  'content_icon'   : 'newsitem_icon.gif',                       # included with Plone
  'product'        : 'PloneNewsItem',
  'factory'        : 'addPloneNewsItem',
  'immediate_view' : 'PloneNewsItem_edit2',
  'filter_content_types' : 0,                                   # This is for folderish types
  'allowed_content_types': (),                                  # If filter_content_types is true, this field can hold allowed meta_types
  'actions': (
    {
    'id'           : 'view',
    'name'         : 'View',
    'action'       : 'string:${object_url}/PloneNewsItem_view',
    'permissions'  : (CMFCorePermissions.View, ),
    'category'      : 'object',
    },
    {
    'id'           : 'edit',
    'name'         : 'Edit',
    'action'       : 'string:${object_url}/PloneNewsItem_editForm2',
    'permissions'   : (CMFCorePermissions.ModifyPortalContent, ),
    'category'      : 'object',
    },
    {
    'id'           : 'image',
    'name'         : 'Images',
    'action'       : 'string:${object_url}/PloneNewsItem_imageForm2',
    'permissions'   : (CMFCorePermissions.ModifyPortalContent, ),
    'category'      : 'object',
    },
    {
    'id'           : 'attach',
    'name'         : 'Attachments',
    'action'       : 'string:${object_url}/PloneNewsItem_attachmentForm2',
    'permissions'   : (CMFCorePermissions.ModifyPortalContent, ),
    'category'      : 'object',
    },
    {
    'id'           : 'model',
    'name'         : 'Model',
    'action'       : 'string:${object_url}/PloneNewsItem_modelForm2',
    'permissions'   : (CMFCorePermissions.ModifyPortalContent, ),
    'category'      : 'object',
    },
    {
    'id'            : 'metadata',
    'name'          : 'Properties',
    'action'        : 'string:${object_url}/metadata_edit_form',
    'permissions'   : (CMFCorePermissions.ModifyPortalContent, ),
    'category'      : 'object',
    },
    {
    'id'            : 'content_status_history',
    'name'          : 'State',
    'action'        : 'string:${object_url}/content_status_history',
    'condition'     : 'python:object and portal.portal_workflow.getTransitionsFor(object, object.getParentNode())',
    'permissions'   : (CMFCorePermissions.View,),
    'category'      : 'object_tabs',
    'visible'       : 0
    },
    ),
  }
  

  
# The usual factory
def addPloneNewsItem(self, id, title = '', REQUEST = {}):
    """
    Factory method for a PloneNewsItem object
    """
    obj = PloneNewsItem(id, title)
    self._setObject(id, obj)

    getattr(self, id)._post_init()
    if REQUEST.has_key('RESPONSE'):
        return REQUEST.RESPONSE.redirect(self.absolute_url() + '/manage_main')

class PloneNewsItem(PortalContent, DefaultDublinCoreImpl, ArticleCore):
    """PloneNewsItem class"""

    #                                                                           #
    #                            BASIC CLASS BEHAVIOUR                          #
    #                                                                           #

    __implements__ = ( PortalContent.__implements__
                     , DefaultDublinCoreImpl.__implements__
                     )

    meta_type = factory_type_information['meta_type']                   # Has not to be the same as FTI but it is the most usual case.

    manage_options = PortalContent.manage_options

    # Standard security settings
    security = ClassSecurityInfo()
    security.declareObjectProtected(CMFCorePermissions.View)            # $$$ Is this clever ? Isn't it better to make the object private ?
                                                                        # This method makes all class properties & methods protected by View by default
    # Init method
    security.declarePrivate('__init__')
    def __init__(self, id, title=''):
        """__init__(self, id, title='')"""
        PloneNewsItem.inheritedAttribute('__init__')(self,)
        ArticleCore.__init__(self)
        self.id = id
        self.title = title
        self.description = ""
        self.text = ""                  # Default text
        self.cooked_text = ""
        self.text_format = "html"

    security.declarePrivate('_post_init')
    def _post_init(self):
        """
        _post_init(self) => Post-init method (that is, method that is called AFTER the class has been set into the ZODB)
        """
        ArticleCore._post_init(self)

    #                                                                           #
    #                           CONTENT EDITING METHODS                         #
    #                                                                           #

    # This edit method should only change attributes that are neither 'id' or metadatas.
    security.declareProtected(PloneNewsItem_editPermission, 'edit')
    def edit(self, text_format, text):
        """
        edit(self, text_format, text) => object modification method
        """
        if html_headcheck(text):
            text=bodyfinder(text)
        self.text = text
        article_tool = getToolByName(self, 'portal_newsitem')
        site_text_formats=article_tool.getSiteTextFormats()

        if len(site_text_formats)==1:
            text_format=site_text_formats[0]  # Has to be the one choice
        elif len(site_text_formats)==0:
            text_format='html'               # Default
        else:
            if text_format in site_text_formats:
                pass # text_format is ok
            else:
                raise "Text format not allowed"

        self.text_format = text_format

        if text_format == 'html':
            self.text = self.cooked_text  = text
        elif text_format == 'plain':
            self.text = text
            self.cooked_text = html_quote(text).replace('\n','<br />')
        elif text_format == 'structured-text':
            self.text = text
            self.cooked_text = format_stx(text)

        # Reindex
        self.reindexObject()

    security.declareProtected(View, 'CookedBody')
    def CookedBody(self):
        """
        The prepared basic rendering of an object. For Documents, this
        means pre-rendered structured text, plain text, or what was between the
        <BODY> tags of HTML.
        """
        if hasattr(self, 'cooked_text'):
            return self.cooked_text
        else:
            return self.text # PloneArticle 1.x did not cook the text

    security.declareProtected(View, 'TextFormat')
    def TextFormat(self):
        """
        Returns the text format set for this Plone Article
        """
        if hasattr(self, 'text_format'):
            if self.text_format in ['html','plain','structured-text']:
                return self.text_format
            else:
                return 'html' # Need a fallback
        else:
            return 'html' # PloneArticle 1.x expected HTML

    #                                                                           #
    #                             CMF CATALOG SUPPORT                           #
    #                                                                           #

    security.declareProtected(View, 'SearchableText')
    def SearchableText(self):
        "Returns a concatination of all searchable text"
        ret = "%s %s %s %s" % (
            self.Title(),
            self.Description(),
            self.text,
            string.join(map(lambda x: x.SearchableText(), self.listAttachments()), " ") +
            string.join(map(lambda x: x.title_or_id(), self.listImages()), " ") 
            )
        return ret

    #                                                                           #
    #                                MISC. METHODS                              #
    #                                                                           #

    security.declarePublic("getArticleURL")
    def getArticleURL(self,):
        """
        getArticleURL(self,) => Return article's absolute URL
        """
        return self.absolute_url()

# CLASS instanciation
InitializeClass(PloneNewsItem)

